#include "Python.h"

#include <utmp.h>


#define OUTVALUE(entry) \
                "(iissssiiiiii)", \
                entry->ut_type, \
                entry->ut_pid, \
                entry->ut_line, \
                entry->ut_id, \
                entry->ut_user, \
                entry->ut_host, \
                entry->ut_exit.e_termination, \
                entry->ut_exit.e_exit, \
                entry->ut_session, \
                entry->ut_tv.tv_sec, \
                entry->ut_tv.tv_usec, \
                entry->ut_addr_v6


static PyObject *
utmp_utmpname(self, args)
    PyObject *self;
    PyObject *args;
{
    char *fname;

    if (!PyArg_ParseTuple(args, "s", &fname))
        return NULL;
    utmpname(fname);
    Py_INCREF(Py_None);
    return Py_None;
}


static PyObject *
utmp_setutent(self, args)
    PyObject *self;
    PyObject *args;
{
    if (!PyArg_ParseTuple(args, ""))
        return NULL;
    setutent();
    Py_INCREF(Py_None);
    return Py_None;
}



static PyObject *
utmp_getutent(self, args)
    PyObject *self;
    PyObject *args;
{
    struct utmp *entry;

    if (!PyArg_ParseTuple(args, ""))
        return NULL;
    entry = getutent();

    if (!entry) {
        Py_INCREF(Py_None);
        return Py_None;
    };

    return Py_BuildValue(OUTVALUE(entry));
}


static PyObject *
utmp_getutid(self, args)  /* args are ut_type=...[, ut_id=...]*/
    PyObject *self;
    PyObject *args;
{
    struct utmp entry;
    struct utmp *nentry;
    short ut_type;
    char *ut_id = "";


    if (!PyArg_ParseTuple(args, "h|s",
            &ut_type, &ut_id
            ))
        return NULL;
    
    entry.ut_type = ut_type;
    strncpy(entry.ut_id, ut_id, 4);
    nentry = getutid(&entry);

    if (!nentry) {
        Py_INCREF(Py_None);
        return Py_None;
    };
    
    return Py_BuildValue(OUTVALUE(nentry));
}


static PyObject *
utmp_getutline(self, args)  /* args is ut_line*/
    PyObject *self;
    PyObject *args;
{
    struct utmp entry;
    struct utmp *nentry;
    char *ut_line;


    if (!PyArg_ParseTuple(args, "s",
            &ut_line
            ))
        return NULL;
    
    strncpy(entry.ut_line, ut_line, UT_LINESIZE);
    nentry = getutline(&entry);

    if (!nentry) {
        Py_INCREF(Py_None);
        return Py_None;
    };
    
    return Py_BuildValue(OUTVALUE(nentry));
}


static PyObject *
utmp_pututline(self, args)
    PyObject *self;
    PyObject *args;
{
    struct utmp entry;
    char *ut_line, *ut_id, *ut_user, *ut_host;

    if (!PyArg_ParseTuple(args, "iissssiiiiii",
            &entry.ut_type, \
            &entry.ut_pid, \
            &ut_line, \
            &ut_id, \
            &ut_user, \
            &ut_host, \
            &entry.ut_exit.e_termination, \
            &entry.ut_exit.e_exit, \
            &entry.ut_session, \
            &entry.ut_tv.tv_sec, \
            &entry.ut_tv.tv_usec, \
            &entry.ut_addr_v6
            ))
        return NULL;

    strncpy(entry.ut_line, ut_line, UT_LINESIZE);
    strncpy(entry.ut_id, ut_id, 4);
    strncpy(entry.ut_user, ut_user, UT_NAMESIZE);
    strncpy(entry.ut_host, ut_host, UT_HOSTSIZE);
    pututline(&entry);
    
    Py_INCREF(Py_None);
    return Py_None;
}



static PyObject *
utmp_endutent(self, args)
    PyObject *self;
    PyObject *args;
{
    if (!PyArg_ParseTuple(args, ""))
        return NULL;
    endutent();
    Py_INCREF(Py_None);
    return Py_None;
}


static PyMethodDef UtmpMethods[] = {
    {"utmpname",   utmp_utmpname,  METH_VARARGS},
    {"setutent",   utmp_setutent,  METH_VARARGS},
    {"getutent",   utmp_getutent,  METH_VARARGS},
    {"endutent",   utmp_endutent,  METH_VARARGS},
    {"getutid",    utmp_getutid,   METH_VARARGS},
    {"getutline",  utmp_getutline, METH_VARARGS},
    {"pututline",  utmp_pututline, METH_VARARGS},
    {NULL,      NULL}        /* Sentinel */
};


void initutmp()
{
    (void) Py_InitModule("utmp", UtmpMethods);
}
